# This file is part of the helm-sign Python package
#    https://gitlab.com/MatthiasLohr/helm-sign
#
# Copyright 2020 Matthias Lohr <mail@mlohr.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from HelmSign.chart import HelmChart


class HelmChartTest(unittest.TestCase):
    def test_init(self):
        chart = HelmChart('tests/charts/hcloud-cloud-controller-manager-2.0.0.tgz')

        # test checksums
        self.assertEqual('6fed28c93d2d45378f528ec5a5d271fd744c49be5e240fdc4d1c6dd3f5119569', chart.checksum('sha256'))

    def test_parse_filename(self):
        self.assertEqual({
            'name': 'hcloud-cloud-controller-manager',
            'version': '2.0.0',
            'major': '2',
            'minor': '0',
            'patch': '0',
            'prerelease': None,
            'build': None
        }, HelmChart.parse_filename('hcloud-cloud-controller-manager-2.0.0.tgz'))



# This file is part of the helm-sign Python package
#    https://gitlab.com/MatthiasLohr/helm-sign
#
# Copyright 2020 Matthias Lohr <mail@mlohr.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest


class End2EndTest(unittest.TestCase):
    def test_sign_verify(self):
        result = os.system(
            'helm-sign --gnupg-home ./tests/keyring tests/charts/hcloud-cloud-controller-manager-2.0.0.tgz'
        )
        self.assertEqual(0, result, 'sign chart')

        result = os.system(
            'helm verify --keyring ./tests/keyring/testkey.pub tests/charts/hcloud-cloud-controller-manager-2.0.0.tgz'
        )
        self.assertEqual(0, result, 'verify signature')



# This file is part of the helm-sign Python package
#    https://gitlab.com/MatthiasLohr/helm-sign
#
# Copyright 2020 Matthias Lohr <mail@mlohr.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import os
import re
import tarfile
from typing import Any, Dict

import semver  # type: ignore
import yaml


class HelmChart(object):
    VERSION_PATTERN = ''.join(semver._REGEX.pattern.split())[1:-1]
    FILENAME_PATTERN = r'^(?P<name>.+)-(?P<version>%s)\.tgz$' % VERSION_PATTERN

    def __init__(self, filename: str) -> None:
        self._filename = filename
        self._chart_info = self.parse_filename(self._filename)

    def checksum(self, hash_alg: str = 'sha256') -> str:
        h = hashlib.new(hash_alg)
        with open(self.filename, 'rb') as f:
            h.update(f.read())
        return h.hexdigest()

    def get_chart_meta(self) -> Any:
        with tarfile.open(self.filename) as archive:
            chart_yaml_fp = archive.extractfile('%s/Chart.yaml' % self._chart_info['name'])
            if chart_yaml_fp is not None:
                meta = yaml.load(chart_yaml_fp, Loader=yaml.SafeLoader)
            else:
                raise RuntimeError('Cannot find Chart.yaml')
        return meta

    @property
    def filename(self) -> str:
        return self._filename

    @staticmethod
    def parse_filename(filename: str) -> Dict[str, str]:
        matches = re.compile(HelmChart.FILENAME_PATTERN).match(os.path.basename(filename))
        if matches is None:
            raise ValueError('Given filename does not seem to be a valid helm chart name')
        return matches.groupdict()



# This file is part of the helm-sign Python package
#    https://gitlab.com/MatthiasLohr/helm-sign
#
# Copyright 2020 Matthias Lohr <mail@mlohr.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os

import gnupg  # type: ignore
import yaml

from .chart import HelmChart


def main() -> int:
    argument_parser = argparse.ArgumentParser()
    argument_parser.add_argument('chart', help='Helm Chart file')
    argument_parser.add_argument('--gnupg-home', help='GnuPG home/keyring directory')
    argument_parser.add_argument('--keyring', help='GnuPG Keyring File')
    argument_parser.add_argument('--key', help='Key to be used for encryption')

    args = argument_parser.parse_args()

    chart = HelmChart(filename=args.chart)

    message = yaml.dump(chart.get_chart_meta())
    message += '\n...\n'
    message += yaml.dump({
        'files': {
            os.path.basename(chart.filename): 'sha256:' + chart.checksum('sha256')
        }
    })

    gpg = gnupg.GPG(gnupghome=args.gnupg_home, keyring=args.keyring)
    signed_data = gpg.sign(message, keyid=args.key)
    with open(chart.filename + '.prov', 'wb') as f:
        f.write(signed_data.data)
    return 0



# This file is part of the helm-sign Python package
#    https://gitlab.com/MatthiasLohr/helm-sign
#
# Copyright 2020 Matthias Lohr <mail@mlohr.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from setuptools import setup, find_packages

with open('README.md', 'r') as readme:
    long_description = readme.read()

setup(
    name='helm-sign',
    description='Gnupg based Signature Tool for Helm Charts',
    long_description=long_description,
    long_description_content_type='text/markdown',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    author='Matthias Lohr',
    author_email='mail@mlohr.com',
    url='https://gitlab.com/MatthiasLohr/helm-sign',
    license='Apache License 2.0',
    install_requires=[
        'python-gnupg==0.4.*',
        'PyYAML==5.3.*',
        'semver==2.9.*'
    ],
    python_requires='>=3.7.*, <4',
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={
    },
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'helm-sign=HelmSign.main:main'
        ]
    },
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    project_urls={
        # 'Documentation': 'https://matthiaslohr.gitlab.io/helm-sign/',
        'Source': 'https://gitlab.com/MatthiasLohr/helm-sign',
        'Tracker': 'https://gitlab.com/MatthiasLohr/helm-sign/issues'
    }
)
